Public Class frmComboBinding

    ''' <summary>
    ''' Erstellt aus 2 generischen Argumenten ein KeyValuePair(Of TKey, TValue).
    ''' Ermglicht quasi krzere Schreibweise des KeyValuePair-Konstruktors
    ''' </summary>
    Public Function BuildPair(Of TKey, TValue)(ByVal Key As TKey, _
        ByVal Value As TValue) As KeyValuePair(Of TKey, TValue)

        Return New KeyValuePair(Of TKey, TValue)(Key, Value)
    End Function

    Private Sub frmComboBinding_Load(ByVal sender As Object, _
        ByVal e As EventArgs) Handles MyBase.Load

        ' "Beschriftete Punkte" erzeugen ...
        Dim Locations = New KeyValuePair(Of String, Point)() { _
            BuildPair("Mid", New Point(150, 75)), _
            BuildPair("TopLeft", New Point(0, 0)), _
            BuildPair("TopRight", New Point(300, 0)), _
            BuildPair("BottomLeft", New Point(0, 150)), _
            BuildPair("BottomRight", New Point(300, 150))}
        ' an Combobox binden ...
        With Me.ComboBox1
            .DisplayMember = "Key"
            .ValueMember = "Value"
            .DataSource = Locations
        End With
        ' ... und an Panel1.Location binden.
        Panel1.DataBindings.Add("Location", Locations, "Value")

        ' Die "beschrifteten Dockstyles" werden mit Linq aus der 
        '  DockStyle-Enumeration generiert
        Dim Tp = GetType(DockStyle)
        Dim Docks As KeyValuePair(Of String, DockStyle)() = _
            [Enum].GetValues(Tp).Cast(Of DockStyle) _
            .Select(Function(DS) BuildPair([Enum].GetName(Tp, DS), DS)) _
            .ToArray()
        ' Einrichtung der Bindungen praktischerweise ausgelagert
        KeyValueBind(Me.ComboBox2, Button1, Docks, "Dock")

        ' Das Generieren "beschrifteter Colors" aus der KnownColor-Enumeration 
        ' ist noch komplizierter, da ich die SystemColors und Color.Transparent 
        ' ausschlieen will (letztere ist als BackColor nicht zulssig).
        Tp = GetType(KnownColor)
        Dim _Colors = [Enum].GetValues(Tp).Cast(Of KnownColor) _
            .Select(Function(KC) BuildPair([Enum].GetName(Tp, KC), _
                Color.FromKnownColor(KC))) _
            .Where(Function(P) Not (P.Value.IsSystemColor OrElse P.Value = _
                Color.Transparent)) _
            .ToArray()
        KeyValueBind(Me.ComboBox3, Panel1, _Colors, "BackColor")
    End Sub

    ''' <summary>
    ''' Verbindet die DataSource mit ListControl und TargetProperty des 
    ''' Target-Controls. Dadurch bernimmt das ListControl die Steuerung der 
    ''' TargetProperty
    ''' </summary>
    Private Sub KeyValueBind( _
            ByVal ListControl As ListControl, _
            ByVal Target As Control, _
            ByVal DataSource As System.Collections.IList, _
            ByVal TargetProperty As String)
        With ListControl
            .DisplayMember = "Key"
            .ValueMember = "Value"
            .DataSource = DataSource
        End With
        Target.DataBindings.Add(TargetProperty, DataSource, "Value")
    End Sub

End Class

